@@ -3,7 +3,7 @@ use std::io::ErrorKind;
 use std::path::PathBuf;
 use structopt::StructOpt;
 
-use preserves_schema::compiler::{CompilerConfig, compile, expand_inputs};
+use preserves_schema::compiler::{ExternalModule, CompilerConfig, compile, expand_inputs};
 
 #[derive(Clone, StructOpt, Debug)]
 struct CommandLine {
@@ -35,7 +35,7 @@ fn main() -> Result<(), Error> {
             (pieces[0], pieces[1])
         };
         let modulepath: Vec<String> = modulepath_str.split('.').map(str::to_owned).collect();
-        config.module_aliases.insert(modulepath, target.to_owned());
+        config.add_external_module(ExternalModule::new(modulepath, target));
     }
     if let Some(c) = args.support_crate {
         config.support_crate = c;
