@@ -7,11 +7,24 @@ pub trait Emittable: std::fmt::Debug {
     fn write_on(&self, f: &mut Formatter);
 }
 
+#[derive(Clone, PartialEq, Eq)]
+pub enum VerticalMode {
+    Variable,
+    Normal,
+    ExtraNewline,
+}
+
+pub trait Vertical {
+    fn set_vertical_mode(&mut self, mode: VerticalMode);
+    fn write_vertically_on(&self, f: &mut Formatter);
+}
+
 pub type Item = std::rc::Rc<dyn Emittable>;
 
 #[derive(Clone)]
 pub struct Sequence {
     pub items: Vec<Item>,
+    pub vertical_mode: VerticalMode,
     pub separator: &'static str,
     pub terminator: &'static str,
 }
@@ -80,6 +93,12 @@ impl Formatter {
     }
 }
 
+impl Default for VerticalMode {
+    fn default() -> Self {
+        Self::Variable
+    }
+}
+
 //---------------------------------------------------------------------------
 
 impl Emittable for &str {
@@ -104,48 +123,86 @@ impl<'a, E: Emittable> Emittable for &'a Vec<E> where &'a E: Emittable {
 
 impl Emittable for Sequence {
     fn write_on(&self, f: &mut Formatter) {
-        let mut need_sep = false;
-        for e in self.items.iter() {
-            if need_sep {
-                self.separator.write_on(f)
-            } else {
-                need_sep = true
+        if self.vertical_mode != VerticalMode::Variable {
+            self.write_vertically_on(f)
+        } else {
+            let mut need_sep = false;
+            for e in self.items.iter() {
+                if need_sep {
+                    self.separator.write_on(f)
+                } else {
+                    need_sep = true
+                }
+                e.write_on(f)
+            }
+            if !self.items.is_empty() {
+                self.terminator.write_on(f)
             }
-            e.write_on(f)
         }
-        self.terminator.write_on(f)
+    }
+}
+
+impl Vertical for Sequence {
+    fn set_vertical_mode(&mut self, vertical_mode: VerticalMode) {
+        self.vertical_mode = vertical_mode;
+    }
+
+    fn write_vertically_on(&self, f: &mut Formatter) {
+        let mut i = self.items.len();
+        let mut first = true;
+        for e in self.items.iter() {
+            if !first {
+                if self.vertical_mode == VerticalMode::ExtraNewline {
+                    f.write("\n");
+                }
+                f.newline();
+            }
+            first = false;
+            e.write_on(f);
+            let delim = if i == 1 { self.terminator } else { self.separator };
+            delim.trim_end_matches(|c: char| c.is_whitespace() && c != '\n').write_on(f);
+            i = i - 1;
+        }
     }
 }
 
 impl Emittable for Grouping {
     fn write_on(&self, f: &mut Formatter) {
-        let mut g = f.copy_empty();
-        self.open.write_on(&mut g);
-        g.write(&self.sequence);
-        self.close.write_on(&mut g);
-        let s = g.buffer;
-        if s.len() <= f.width {
-            f.write(&s)
+        if self.sequence.vertical_mode != VerticalMode::Variable {
+            self.write_vertically_on(f)
         } else {
-            self.open.write_on(f);
-            if !self.sequence.items.is_empty() {
-                f.with_indent(|f| {
-                    let mut i = self.sequence.items.len();
-                    for e in self.sequence.items.iter() {
-                        f.newline();
-                        e.write_on(f);
-                        let delim = if i == 1 { self.sequence.terminator } else { self.sequence.separator };
-                        delim.trim_end().write_on(f);
-                        i = i - 1;
-                    }
-                });
-                f.newline()
+            let mut g = f.copy_empty();
+            self.open.write_on(&mut g);
+            g.write(&self.sequence);
+            self.close.write_on(&mut g);
+            let s = g.buffer;
+            if s.len() <= f.width {
+                f.write(&s)
+            } else {
+                self.write_vertically_on(f)
             }
-            self.close.write_on(f);
         }
     }
 }
 
+impl Vertical for Grouping {
+    fn set_vertical_mode(&mut self, vertical_mode: VerticalMode) {
+        self.sequence.set_vertical_mode(vertical_mode);
+    }
+
+    fn write_vertically_on(&self, f: &mut Formatter) {
+        self.open.write_on(f);
+        if !self.sequence.items.is_empty() {
+            f.with_indent(|f| {
+                f.newline();
+                self.sequence.write_vertically_on(f)
+            });
+            f.newline()
+        }
+        self.close.write_on(f);
+    }
+}
+
 impl<'a, E: Emittable> Emittable for &'a E {
     fn write_on(&self, f: &mut Formatter) {
         (*self).write_on(f)
@@ -210,21 +267,23 @@ pub mod constructors {
     use super::Grouping;
     use super::Item;
     use super::Emittable;
+    use super::VerticalMode;
+    use super::Vertical;
 
     pub fn item<E: 'static + Emittable>(i: E) -> Item {
         std::rc::Rc::new(i)
     }
 
     pub fn name(pieces: Vec<Item>) -> Sequence {
-        Sequence { items: pieces, separator: "::", terminator: "" }
+        Sequence { items: pieces, vertical_mode: VerticalMode::default(), separator: "::", terminator: "" }
     }
 
     pub fn seq(items: Vec<Item>) -> Sequence {
-        Sequence { items: items, separator: "", terminator: "" }
+        Sequence { items: items, vertical_mode: VerticalMode::default(), separator: "", terminator: "" }
     }
 
     pub fn commas(items: Vec<Item>) -> Sequence {
-        Sequence { items: items, separator: ", ", terminator: "" }
+        Sequence { items: items, vertical_mode: VerticalMode::default(), separator: ", ", terminator: "" }
     }
 
     pub fn parens(items: Vec<Item>) -> Grouping {
@@ -245,64 +304,92 @@ pub mod constructors {
 
     pub fn block(items: Vec<Item>) -> Grouping {
         Grouping {
-            sequence: Sequence { items: items, separator: " ", terminator: "" },
+            sequence: Sequence {
+                items: items,
+                vertical_mode: VerticalMode::default(),
+                separator: " ",
+                terminator: "",
+            },
             open: "{",
             close: "}",
         }
     }
 
+    pub fn codeblock(items: Vec<Item>) -> Grouping {
+        vertical(false, block(items))
+    }
+
     pub fn semiblock(items: Vec<Item>) -> Grouping {
         Grouping {
-            sequence: Sequence { items: items, separator: "; ", terminator: "" },
+            sequence: Sequence {
+                items: items,
+                vertical_mode: VerticalMode::default(),
+                separator: "; ",
+                terminator: "",
+            },
             open: "{",
             close: "}",
         }
     }
+
+    pub fn vertical<V: Vertical>(spaced: bool, mut v: V) -> V {
+        v.set_vertical_mode(if spaced { VerticalMode::ExtraNewline } else { VerticalMode::Normal });
+        v
+    }
+
+    pub fn indented(sequence: Sequence) -> Grouping {
+        Grouping { sequence, open: "", close: "" }
+    }
 }
 
 pub mod macros {
     #[macro_export]
     macro_rules! name {
-        ($($item:expr),*) => {crate::syntax::block::constructors::name(vec![$(std::rc::Rc::new($item)),*])}
+        ($($item:expr),*) => {$crate::syntax::block::constructors::name(vec![$(std::rc::Rc::new($item)),*])}
     }
 
     #[macro_export]
     macro_rules! seq {
-        ($($item:expr),*) => {crate::syntax::block::constructors::seq(vec![$(std::rc::Rc::new($item)),*])}
+        ($($item:expr),*) => {$crate::syntax::block::constructors::seq(vec![$(std::rc::Rc::new($item)),*])}
     }
 
     #[macro_export]
     macro_rules! commas {
-        ($($item:expr),*) => {crate::syntax::block::constructors::commas(vec![$(std::rc::Rc::new($item)),*])}
+        ($($item:expr),*) => {$crate::syntax::block::constructors::commas(vec![$(std::rc::Rc::new($item)),*])}
     }
 
     #[macro_export]
     macro_rules! parens {
-        ($($item:expr),*) => {crate::syntax::block::constructors::parens(vec![$(std::rc::Rc::new($item)),*])}
+        ($($item:expr),*) => {$crate::syntax::block::constructors::parens(vec![$(std::rc::Rc::new($item)),*])}
     }
 
     #[macro_export]
     macro_rules! brackets {
-        ($($item:expr),*) => {crate::syntax::block::constructors::brackets(vec![$(std::rc::Rc::new($item)),*])}
+        ($($item:expr),*) => {$crate::syntax::block::constructors::brackets(vec![$(std::rc::Rc::new($item)),*])}
     }
 
     #[macro_export]
     macro_rules! anglebrackets {
-        ($($item:expr),*) => {crate::syntax::block::constructors::anglebrackets(vec![$(std::rc::Rc::new($item)),*])}
+        ($($item:expr),*) => {$crate::syntax::block::constructors::anglebrackets(vec![$(std::rc::Rc::new($item)),*])}
     }
 
     #[macro_export]
     macro_rules! braces {
-        ($($item:expr),*) => {crate::syntax::block::constructors::braces(vec![$(std::rc::Rc::new($item)),*])}
+        ($($item:expr),*) => {$crate::syntax::block::constructors::braces(vec![$(std::rc::Rc::new($item)),*])}
     }
 
     #[macro_export]
     macro_rules! block {
-        ($($item:expr),*) => {crate::syntax::block::constructors::block(vec![$(std::rc::Rc::new($item)),*])}
+        ($($item:expr),*) => {$crate::syntax::block::constructors::block(vec![$(std::rc::Rc::new($item)),*])}
+    }
+
+    #[macro_export]
+    macro_rules! codeblock {
+        ($($item:expr),*) => {$crate::syntax::block::constructors::codeblock(vec![$(std::rc::Rc::new($item)),*])}
     }
 
     #[macro_export]
     macro_rules! semiblock {
-        ($($item:expr),*) => {crate::syntax::block::constructors::semiblock(vec![$(std::rc::Rc::new($item)),*])}
+        ($($item:expr),*) => {$crate::syntax::block::constructors::semiblock(vec![$(std::rc::Rc::new($item)),*])}
     }
 }
